/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkComplexBlackBoxOutputDescriptor.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:12 $
  Version:   $Revision: 1.4 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

/**
 *  \file 
 *  \brief Class bbtk::ComplexBlackBoxOutputDescriptor : structure containing the description of a ComplexBlackBox output (name, description, type, mapping)
 */
/**
 * \class bbtk::ComplexBlackBoxOutputDescriptor
 * \brief Structure containing the description of a ComplexBlackBox output (name, description, type, mapping)
 */

#ifndef __bbtkComplexBlackBoxOutputDescriptor_h__
#define __bbtkComplexBlackBoxOutputDescriptor_h__

#include "bbtkBlackBoxOutputDescriptor.h"
#include "bbtkMessageManager.h"
//#include "bbtkSystem.h"

namespace bbtk
{


  class BBTK_EXPORT ComplexBlackBoxOutputDescriptor : 
    public bbtk::BlackBoxOutputDescriptor
  {
  public:
      
    ComplexBlackBoxOutputDescriptor( TypeInfo creator_type_info,
				     const std::string& name,
				     const std::string& description,
				     const std::string& nature,
				     const std::string& target,
				     const std::string& output,
				     TypeInfo type);
    
    ~ComplexBlackBoxOutputDescriptor();
    
    /// Returns the type of the output
    TypeInfo GetTypeInfo() const { return mType; }
    /// Returns the name of the type of the output    
    std::string GetTypeName() const { return TypeName(mType); }
    /// Returns the name of the type of the output    
    std::string GetHumanTypeName() const { return HumanTypeName(mType); }
    /// 
    bool IsPointerType() const { return false; }
    

    /// Returns the target black box 
    const std::string& GetTarget() const { return mTarget; }
    /// Returns the output 
    const std::string& GetOutput() const { return mOutput; }
        
  private:
    /// Default ctor is private 
    ComplexBlackBoxOutputDescriptor() : 
      BlackBoxOutputDescriptor(typeid(void),"","",""), 
      mType(typeid(void))  {}
    std::string mTarget;
    std::string mOutput;
    TypeInfo mType;
  };

}
// namespace bbtk
#endif
