/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbvtkMIPCreator.cxx,v $
  Language:  C++
  Date:      $Date: 2009/05/15 14:58:01 $
  Version:   $Revision: 1.6 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

/**
 *  \file 
 *  \brief 
 */



#ifdef _USE_VTK_


#include "bbvtkMIPCreator.h"
#include "bbvtkPackage.h"

namespace bbvtk
{


   BBTK_ADD_BLACK_BOX_TO_PACKAGE(vtk,MIPCreator)
   BBTK_BLACK_BOX_IMPLEMENTATION(MIPCreator,bbtk::AtomicBlackBox);





	//---------------------------------------------------------------------
   void MIPCreator::bbUserSetDefaultValues() 
   { 
	   bbSetInputShift(0);
	   bbSetInputScale(1.0);
	   mCast	= NULL;  
	   mMIP		= NULL;  
	   mMapper	= NULL;  
	   mVolume	= NULL;  
   }
	
	//---------------------------------------------------------------------
   void MIPCreator::bbUserInitializeProcessing() 
   {
	   
    // Create the pipeline
    mCast = vtkImageShiftScale::New();
    mCast->SetOutputScalarTypeToUnsignedChar();
    mCast->ClampOverflowOn();
    
    mMIP = vtkVolumeRayCastMIPFunction::New();
    mMIP->SetMaximizeMethodToScalarValue();
    
    mMapper = vtkVolumeRayCastMapper::New();
    mMapper->SetVolumeRayCastFunction(mMIP);
    mMapper->SetInput(mCast->GetOutput()); // (smoother.GetOutput())
    
    mVolume = vtkVolume::New();
    mVolume->SetMapper(mMapper);

    //  mMapper->ScalarVisibilityOff();
    //    mMapper->ImmediateModeRenderingOn();

    bbSetOutputOut(mVolume);
}

	//---------------------------------------------------------------------
   void MIPCreator::bbUserFinalizeProcessing() 
   { 
	   if (mCast!=NULL)
	   {
		   mCast->Delete();
		   mCast=NULL;
	   }

	   if (mMIP!=NULL)
	   {
		   mMIP->Delete();
		   mMIP=NULL;
	   }

	   if (mMapper!=NULL)
	   {
		   mMapper->Delete();
		   mMapper=NULL;
	   }

	   if (mVolume!=NULL)
	   {
		   mVolume->Delete();
		   mVolume=NULL;
	   }
	   

   }
//---------------------------------------------------------------------

   void MIPCreator::Process()
   {
     mCast->SetInput( bbGetInputIn() );
     mCast->SetScale( bbGetInputScale() / 100.0 );
     mCast->SetShift( -bbGetInputShift() );

     mMapper->Update();
     
     //     mVolume->GetProperty()->SetColor( bbGetInputColour()[0],  bbGetInputColour()[1], bbGetInputColour()[2] );
     //     mVolume->GetProperty()->SetOpacity( bbGetInputOpacity() );
     
   }
}//namespace bbtk

#endif // _USE_VTK_

